local revision = tonumber(("$Revision: 55609 $"):match("%d+"))
if not NDI_REVISION or revision > NDI_REVISION then
    NDI_REVISION = revision
    NDI_DATE = ("$Date: 2007-11-21 09:01:09 -0500 (Wed, 21 Nov 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = AceLibrary("AceLocale-2.2"):new("NDragIt")
L:RegisterTranslations("enUS", function() return {
    ["Add"] = true,
    ["Add a new frame to the set of draggable frames."] = true,
    ["Frame"] = true,
    ["Add a frame to be made draggable."] = true,
    ["Child Frame"] = true,
    ["Add a child frame whose parent is to be made draggable."] = true,
    ["LOD Frame"] = true,
    ["Add a frame to be made draggable, which is part of a LOD addon."] = true,
    ["List"] = true,
    ["List all frames that are draggable."] = true,
    ["Delete"] = true,
    ["Delete a frame from the set of draggable frames."] = true,
    ["Delete draggability for %s."] = true,
    
    ["%s added."] = true,
    ["%s deleted."] = true,
    
    ["Error: %s not found."] = true,
    ["Error: %s is not a frame."] = true,
    ["Error: %s must have UIParent as its immediate parent."] = true,
    ["Error: %s already present."] = true,
    ["Error: parent is not a frame."] = true,
    ["Error: parent must have UIParent as its immediate parent."] = true,
    ["Error: incorrect usage."] = true,
    
    ["Frames:"] = true,
    ["Child Frames:"] = true,
    ["LOD Frames:"] = true,
    ["LOD Child Frames:"] = true,
} end)

local VERSION = tonumber(("$Revision: 55609 $"):match("%d+"))
NDragIt = AceLibrary("AceAddon-2.0"):new("AceHook-2.1", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

local FRAMES = {
    -- default UI
    "SpellBookFrame",
    "QuestLogFrame",
    "FriendsFrame",
    "LFGParentFrame",
    "KnowledgeBaseFrame",
    "HelpFrame",
    "GossipFrame",
    "MerchantFrame",
    "MailFrame",
    "OpenMailFrame",
    "GuildRegistrarFrame",
    "PetPaperDollFrame",
    "DressUpFrame",
    "TabardFrame",
    "TaxiFrame",
    "QuestFrame",
    "TradeFrame",
    "LootFrame",
    "PetStableFrame",
    "GroupLootFrame1",
    "GroupLootFrame2",
    "GroupLootFrame3",
    "GroupLootFrame4",
    "StackSplitFrame",
    "PetitionFrame",
    "WorldStateScoreFrame",
    "BattlefieldFrame",
    "ArenaFrame",
    "ItemTextFrame",
    -- addons
    "LudwigFrame",
    "LootLinkFrame", "LootLinkSearchFrame",
}
local CHILD_FRAMES = {
    -- default UI
    "PaperDollFrame", "ReputationFrame", "SkillFrame", "PVPFrame",
    "InspectPVPFrame", "InspectTalentFrame",
	"SendMailFrame",
}
local LOD_FRAMES = {
    -- default UI
    Blizzard_AuctionUI = "AuctionFrame",
    Blizzard_BindingUI = "KeyBindingFrame",
    Blizzard_CraftUI = "CraftFrame",
    Blizzard_GMSurveyUI = "GMSurveyFrame",
    Blizzard_InspectUI = "InspectFrame",
    Blizzard_ItemSocketingUI = "ItemSocketingFrame",
    Blizzard_MacroUI = "MacroFrame",
    Blizzard_TalentUI = "TalentFrame",
    Blizzard_TradeSkillUI = "TradeSkillFrame",
    Blizzard_TrainerUI = "ClassTrainerFrame",
    Blizzard_GuildBankUI = "GuildBankFrame",
    -- addons
    DBM_GUI = "DBMBossModFrame",
}

ndfDebug_g = false
ndfShowFrame_g = false

NDragIt.lodLoaded_m = {}

function NDragIt:OnInitialize()
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:OnInitialize") end
    
    self.version = "r"..NDI_REVISION
    self.date = NDI_DATE
    
    self:SetupDatabase()
    
    self:RegisterChatCommand({"/ndragit", "/dragit"}, self.options)
end

function NDragIt:OnEnable(first)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:OnEnable first = "..tostring(first)) end

    for i, v in ipairs(self.db.profile.frames) do
        self:ActivateFrame(v)
    end
    for i, v in ipairs(self.db.profile.childFrames) do
        self:ActivateChildFrame(v)
    end
    local frame = nil
    for k, v in pairs(self.db.profile.lodFrames) do
        frame = getglobal(v)
        if frame then
            self:ADDON_LOADED(k)
        end
    end
    
    self:BuildDeleteList()
    
    self:RegisterEvent("ADDON_LOADED")
end

function NDragIt:OnDisable()
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:OnEnable first = "..tostring(first)) end
    
    self.lodLoaded_m = {}
end

function NDragIt:OnProfileEnable()
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:OnProfileEnable") end
    
    self:UnhookAll()
    self:UnregisterAllEvents()
    
    self:LoadDefaultFrames()
    
    self.lodLoaded_m = {}
    
    if self:IsActive() then
        self:OnEnable()
    end
end

function NDragIt:ADDON_LOADED(addon)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:ADDON_LOADED "..tostring(addon)) end
    
    local frame = self.db.profile.lodFrames[addon]
    if frame then
        if not (self.lodLoaded_m[addon]) then
            self.lodLoaded_m[addon] = frame
            self:ActivateFrame(frame)
        end
    end
end

function NDragIt:ActivateFrame(name)
    frame = getglobal(name)
    if frame then
        if ndfDebug_g then ChatFrame1:AddMessage(" frame: "..name) end
        frame:EnableMouse(true)
        frame:SetMovable(true)
        self:HookScript(frame, "OnMouseDown", "OnMouseDown")
        self:HookScript(frame, "OnMouseUp", "OnMouseUp")
    end
end

function NDragIt:ActivateChildFrame(name)
    frame = getglobal(name)
    if frame then
        local parent = frame:GetParent()
        if parent then
            if ndfDebug_g then ChatFrame1:AddMessage(" child frame: "..name) end
            frame:EnableMouse(true)
            parent:SetMovable(true)
            self:HookScript(frame, "OnMouseDown", "ChildOnMouseDown")
            self:HookScript(frame, "OnMouseUp", "ChildOnMouseUp")
        end
    end
end

function NDragIt:OnMouseDown(frame, button)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:OnMouseDown") end

    if button == "LeftButton" then
        if frame then
            if ndfShowFrame_g then
                frame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", })
            end
            frame:StartMoving()
            frame:SetUserPlaced(false)
        end
    end
	
	self.hooks[frame]["OnMouseDown"](frame, button)
end

function NDragIt:OnMouseUp(frame, button)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:OnMouseUp") end

    if frame then
        if ndfShowFrame_g then
            frame:SetBackdrop({})
        end
        frame:StopMovingOrSizing()
    end
	
	self.hooks[frame]["OnMouseUp"](frame, button)
end

function NDragIt:ChildOnMouseDown(frame, button)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:ChildOnMouseDown") end

    if button == "LeftButton" then
        if frame then
            local parentFrame = frame:GetParent()
            if parentFrame then
                if ndfShowFrame_g then
                    frame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", })
                end
                parentFrame:StartMoving()
                parentFrame:SetUserPlaced(false)
            end
        end
    end
	
	self.hooks[frame]["OnMouseDown"](frame, button)
end

function NDragIt:ChildOnMouseUp(frame, button)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:ChildOnMouseUp") end

    if frame then
        local parentFrame = frame:GetParent()
        if parentFrame then
            if ndfShowFrame_g then
                frame:SetBackdrop({})
            end
            parentFrame:StopMovingOrSizing()
        end
    end
	
	self.hooks[frame]["OnMouseUp"](frame, button)
end

function NDragIt:AddFrame(argstr)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:AddFrame "..tostring(argstr)) end
    
    if self:FramePresent(argstr) then
        return
    end
    if not self:ValidFrame(argstr) then
        return
    end
    table.insert(self.db.profile.frames, argstr)
    self:Print(string.format(L["%s added."], argstr))
    self:ActivateFrame(argstr)
    self:BuildDeleteList()
end

function NDragIt:AddChildFrame(argstr)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:AddChildFrame "..tostring(argstr)) end
    
    if self:FramePresent(argstr) then
        return
    end
    if not self:ValidChildFrame(argstr) then
        return
    end
    table.insert(self.db.profile.childFrames, argstr)
    self:Print(string.format(L["%s added."], argstr))
    self:ActivateChildFrame(argstr)
    self:BuildDeleteList()
end

function NDragIt:AddLODFrame(argstr)
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:AddLODFrame "..tostring(argstr)) end

    local addon, frame = nil, nil
    for k, v in argstr:gmatch('%s*(%S+)%s+(.+)') do
        frame, addon = k, v
    end
    if not addon or not frame then
        self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", L["Error: incorrect usage."])
        return
    end
    if self:FramePresent(frame, addon) then
        return
    end
    if not self:ValidFrame(frame) then
        return
    end
    self.db.profile.lodFrames[addon] = frame
    self:Print(string.format(L["%s added."], argstr))
    self:ActivateFrame(frame)
    self:BuildDeleteList()
end

function NDragIt:ValidFrame(name)
    local frame = getglobal(name)
    if not frame then
        self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s not found."], name))
        return false
    end
    if not frame.GetObjectType or frame:GetObjectType() ~= "Frame" then
        self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s is not a frame."], name))
        return false
    end
    local parent = frame:GetParent()
    if not parent or not parent.GetName or parent:GetName() ~= "UIParent" then
        self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s must have UIParent as its immediate parent."], name))
        return false
    end
    return true
end

function NDragIt:ValidChildFrame(name)
    local frame = getglobal(name)
    if not frame then
        self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s not found."], name))
        return false
    end
    if not frame.GetObjectType or frame:GetObjectType() ~= "Frame" then
        self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s is not a frame."], name))
        return false
    end
    local parent = frame:GetParent()
    if not parent or not parent.GetObjectType or parent:GetObjectType() ~= "Frame" then
        self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", L["Error: parent is not a frame."])
        return false
    end
    local grandParent = parent:GetParent()
    if not grandParent or not grandParent.GetName or grandParent:GetName() ~= "UIParent" then
        self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", L["Error: parent must have UIParent as its immediate parent."])
        return false
    end
    
    return true
end

function NDragIt:FramePresent(frame, addon)
    for i, v in ipairs(self.db.profile.frames) do
        if frame == v then
            self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s already present."], frame))
            return true
        end
    end
    for i, v in ipairs(self.db.profile.childFrames) do
        if frame == v then
            self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s already present."], frame))
            return true
        end
    end
    for k, v in pairs(self.db.profile.lodFrames) do
        if addon == k then
            self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s already present."], addon))
            return true
        end
        if frame == v then
            self:CustomPrint(1, 0, 0, DEFAULT_CHAT_FRAME, nil, "", string.format(L["Error: %s already present."], frame))
            return true
        end
    end

    return false
end

function NDragIt:List()
    self:Print(L["LOD Frames:"])
    for k, v in pairs(self.db.profile.lodFrames) do
        self:Print(string.format("  %s: %s", tostring(k), tostring(v)))
    end
    self:Print(L["Child Frames:"])
    for i, v in ipairs(self.db.profile.childFrames) do
        self:Print(string.format("  %s", tostring(v)))
    end
    self:Print(L["Frames:"])
    for i, v in ipairs(self.db.profile.frames) do
        self:Print(string.format("  %s", tostring(v)))
    end
end

function NDragIt:BuildDeleteList()
    if ndfDebug_g then ChatFrame1:AddMessage("NDragIt:BuildDeleteList "..tostring(argstr)) end
    
    local index = 101
    self.options.args.delete.args = {}
    for i, v in ipairs(self.db.profile.frames) do
        self.options.args.delete.args[v] = {
            name = v,
            desc = string.format(L["Delete draggability for %s."], v),
            type = "execute",
            func = function(frame) NDragIt:DeleteFrame(frame) end,
            passValue = v,
            order = index,
        }
        index = index + 1
    end
    for i, v in ipairs(self.db.profile.childFrames) do
        self.options.args.delete.args[v] = {
            name = v,
            desc = string.format(L["Delete draggability for %s."], v),
            type = "execute",
            func = function(frame) NDragIt:DeleteChildFrame(frame) end,
            passValue = v,
            order = index,
        }
        index = index + 1
    end
    for k, v in pairs(self.db.profile.lodFrames) do
        self.options.args.delete.args[k] = {
            name = k,
            desc = string.format(L["Delete draggability for %s."], k),
            type = "execute",
            func = function(addon) NDragIt:DeleteLodFrame(addon) end,
            passValue = k,
            order = index,
        }
        index = index + 1
    end
end

function NDragIt:DeleteFrame(name)
    local index = nil
    for i, v in ipairs(self.db.profile.frames) do
        if v == name then
            index = i
            break
        end
    end
    table.remove(self.db.profile.frames, index)
    local frame = getglobal(name)
    if frame then
        self:Unhook(frame, "OnMouseDown")
        self:Unhook(frame, "OnMouseUp")
    end
    self:BuildDeleteList()
    self:Print(string.format(L["%s deleted."], name))
end

function NDragIt:DeleteChildFrame(name)
    local index = nil
    for i, v in ipairs(self.db.profile.childFrames) do
        if v == name then
            index = i
            break
        end
    end
    table.remove(self.db.profile.childFrames, index)
    local frame = getglobal(name)
    if frame then
        self:Unhook(frame, "OnMouseDown")
        self:Unhook(frame, "OnMouseUp")
    end
    self:BuildDeleteList()
    self:Print(string.format(L["%s deleted."], name))
end

function NDragIt:DeleteLodFrame(addon)
    local index = nil
    local frame = getglobal(self.db.profile.lodFrames[addon])
    if frame then
        self:Unhook(frame, "OnMouseDown")
        self:Unhook(frame, "OnMouseUp")
    end
    self.db.profile.lodFrames[addon] = nil
    self:BuildDeleteList()
    self:Print(string.format(L["%s deleted."], addon))
end

function NDragIt:SetupDatabase()
    self:RegisterDB("NDragItDB")
    self:RegisterDefaults("profile", {
        frames = nil,
        childFrames = nil,
        lodFrames = nil,
    })
    
    local profile = self.db.profile
    if not profile.frames and not profile.childFrames and not profile.lodFrames then
        self:LoadDefaultFrames()
    end
end

function NDragIt:LoadDefaultFrames()
    local profile = self.db.profile
    profile.frames = {}
    for i, v in ipairs(FRAMES) do
        table.insert(profile.frames, v)
    end
    profile.childFrames = {}
    for i, v in ipairs(CHILD_FRAMES) do
        table.insert(profile.childFrames, v)
    end
    profile.lodFrames = {}
    for k, v in pairs(LOD_FRAMES) do
        profile.lodFrames[k] = v
    end
end

NDragIt.options = {
    type = "group",
    args = {
        add = {
            name = L["Add"],
            desc = L["Add a new frame to the set of draggable frames."],
            type = "group",
            disabled = "~IsActive",
            args = {
                frame = {
                    name = L["Frame"],
                    desc = L["Add a frame to be made draggable."],
                    type = "text",
                    usage = "frame <frameName> e.g.\n/dragit add frame QuestLogFrame",
                    set = function(argstr) NDragIt:AddFrame(argstr) end,
                    get = false,
                    order = 101,
                },
                childFrame = {
                    name = L["Child Frame"],
                    desc = L["Add a child frame whose parent is to be made draggable."],
                    type = "text",
                    usage = "childFrame <frameName> e.g.\n/dragit add childFrame ReputationFrame",
                    set = function(argstr) NDragIt:AddChildFrame(argstr) end,
                    get = false,
                    order = 102,
                },
                lodFrame = {
                    name = L["LOD Frame"],
                    desc = L["Add a frame to be made draggable, which is part of a LOD addon."],
                    type = "text",
                    usage = "lodFrame <frameName> <addonName>\ne.g. /dragit add lodFrame DBMBossModFrame DBM_GUI",
                    set = function(argstr) NDragIt:AddLODFrame(argstr) end,
                    get = false,
                    order = 103,
                },
            },
            order = 101,
        },
        list = {
            name = L["List"],
            desc = L["List all frames that are draggable."],
            type = "execute",
            disabled = "~IsActive",
            func = NDragIt.List,
            passValue = NDragIt,
            order = 102,
        },
        delete = {
            name = L["Delete"],
            desc = L["Delete a frame from the set of draggable frames."],
            type = "group",
            disabled = "~IsActive",
            args = {
            },
            order = 103,
        },
    },
}
